Visual Recognition Library (VRL)
================================

This library contains the code used in:

  Jorge Sánchez and Javier Redolfi, "Exponential family Fisher vector for image
  classification". Pattern Reconition Letters, 2015 (ib-press)

0. Features
---

The library consist basically of the following python modules:

 * The computer vision module (cv): image handling and multiscale features
 * The machine learning module (ml): mixture models, exponential family Fisher
   vectors, sklearn-based objects, etc.
 * Datasets (datasets): train/test sets generation and evaluation

1. Dependencies
---

* cmake >= 2.8
* gcc >= 4.8
* eigen >= 3.2 -> http://eigen.tuxfamily.org
* openmp -> http://openmp.org/wp/
* vlfeat -> http://www.vlfeat.org/
* python 2.7
* scipy
* numpy
* joblib
* scikit-learn -> scikit-learn.org/
* scikit-image -> http://scikit-image.org/
* python-nose (optional)

2. Installation on Debian/Ubuntu
---

**2.1. Install dependencies**

> \# apt-get install python2.7 cmake build-essential libgomp1 libeigen3-dev
  python-scipy python-numpy python-skimage python-sklearn python-joblib python-nose

Optionally, you might also need virtualenv:

> \# apt-get install python-virtualenv

**2.2. Build the module**

Get the package:

> $ wget http://www.famaf.unc.edu.ar/~jsanchez/efv/vrl-0.1.tar.gz

> $ tar -xf vrl-0.1.tar.gz

> $ cd vrl-0.1

Create a virtualenv and use the installed versions of scipy, numpy, etc.:

> $ virtualenv venv --system-site-packages

> $ source venv/bin/activate

Build the library. If you have vlfeat on your system, you can set a VLFEAT_PATH
env variable (e.g. export VLFEAT_PATH=/path/to/vlfeat) before the cmake line
and vrl will be compiled against it. Otherwise, it try to get it from the
project website.

> (venv)$ mkdir build; cd build

> (venv)$ cmake ../

> (venv)$ make

> (venv)$ make install

> (venv)$ cd ../

**2.3 Run some tests to check that everithing is OK**

> (venv)$ nosetests tests

If you don't use virtualenvs, you'll need to set the LD_LIBRARY_PATH variable
 to the path where the shared libs (libcv.so, libml.so) were installed. For
 example, if you want to install vrl locally (at user level):

> $ mkdir build; cd build

> $ cmake -DUSER_INSTALL=1 ../  # runs "python setup.py install --user"

> $ make; make install

> $ cd ../

> $ export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$HOME/.local/lib:/usr/local/lib

> $ nosetest tests

You can also run cmake without the USER_INSTALL option. In that case, you'll
need root provilegies to install it.


3. Classification example on KTH-TIPS2-a
----------------------------------------

Get the dataset:

> (venv)$ wget http://www.nada.kth.se/cvap/databases/kth-tips/kth-tips2-a_col_200x200.tar

> (venv)$ tar -xf kth-tips2-a_col_200x200.tar

Edit $VRL/bin/classification_demo_kth_tips_2a.py and set "DATASET\_PATH" and
"OUTPUT\_PATH" accordingly. The former is the path where all intermetiate files
(features, models, etc.) will be stored.

Run the demo and wait for the results:

> (venv)$ python bin/classification_demo_kth_tips_2a.py

The demo runs a standard SIFT-based pipeline on the KTH-TIPS2-a dataset, using
sample "a" for evaluation. You can comment/uncomment the last two lines in the
file to run experiments using LBPs and a BMM-based eFV.
