#ifndef LIB_CORE_MEMORY_H_
#define LIB_CORE_MEMORY_H_

#include <cstdint>

#if defined(__SSE2__)
#include <xmmintrin.h>
#else
#error "SSE2 not supported"
#endif

#include "core.h"

static const uint32_t ALIGNMENT = 16;  /**< in bytes */
static const uint32_t ALIGNMENT_MINUS_ONE = ALIGNMENT - 1;
static const uint32_t MIN_ALLOWED_SIZE = 64;

template<typename T>
inline static T* Allocate(uint32_t numel) {
#if defined(__SSE2__)
  return static_cast<T*>(_mm_malloc(numel * sizeof(T), ALIGNMENT));
#else
  return static_cast<T*>(malloc(numel * sizeof(T)));
#endif
}

template<typename T>
inline static void Deallocate(T *p) {
#if defined(__SSE2__)
  _mm_free(static_cast<void*>(p));
#else
  free(static_cast<void*>(p));
#endif
}

template<typename T>
inline static bool isAligned(T *p) {
  return ((reinterpret_cast<size_t>(p) & ALIGNMENT_MINUS_ONE) == 0);
}

#endif  // LIB_CORE_MEMORY_H_
