#include "hog.h"

extern "C" {
#include <vl/hog.h>
}

#include <iostream>

#include <cmath>
#include <cstring>

#include "../core/core.h"
#include "../core/memory.h"
#include "../core/vecmath.h"

HOG::HOG(int num_orientations, int cell_size, int norm)
    :num_orientations_(num_orientations)
    , cell_size_(cell_size)
    , norm_(norm) {
  assert(num_orientations > 0);
  assert(cell_size > 0);

  hog_ = NULL;
}

HOG::~HOG() {
  if (hog_ != NULL) {
    vl_hog_delete((VlHog *)hog_);
  }
}

int
HOG::Process(float *img, int w, int h) {
  if (hog_ == NULL ) {
    hog_ = (void *)vl_hog_new(VlHogVariantUoctti, num_orientations_, false);
    //hog_ = (void *)vl_hog_new(VlHogVariantDalalTriggs, num_orientations_, false);
  }

  vl_hog_put_image((VlHog *)hog_, img, h, w, 1, cell_size_) ;

  return this->ndim();
}

void
HOG::Extract(float *arr) {
  assert (hog_ != NULL);

  const int ndim_ = this->ndim();

  vl_hog_extract((VlHog *)hog_, arr) ;

  if (norm_ < 0) {
    return;
  }
  NormalizeLp(ndim_, arr, norm_);
}

int
HOG::ndim(void) const {
  if (hog_ == NULL) {
    return 0;
  }

  const int hog_w = vl_hog_get_width((VlHog *)hog_);
  const int hog_h = vl_hog_get_height((VlHog *)hog_);
  const int hog_d = vl_hog_get_dimension((VlHog *)hog_);

  return (hog_w * hog_h * hog_d);
}

#ifdef _PYTHON
HOG* HOG_new(int num_orientations, int cell_size, int norm) {
  return new HOG(num_orientations, cell_size, norm);
}

void HOG_del(HOG* obj) {
  delete obj;
}

int HOG_Process(HOG* obj, float *img, int w, int h) {
  return obj->Process(img, w, h);
}

int HOG_ndim(HOG* obj) {
  return obj->ndim();
}

void HOG_Extract(HOG* obj, float *arr) {
  obj->Extract(arr);
}
#endif
