#ifndef LIB_CV_HOG_
#define LIB_CV_HOG_
/**
 * @class HOG
 *  dense SIFT features based on vlfeat's HOG
 *
 * @date  21-Jan-2014
 */
class HOG {
 public:
  /**
   * Constructor
   *
   * @param num_orientations
   * @param cell_size
   * @param norm               Lp-normalization (0: None, 1: L1, 2: L2)
   *
   */
  HOG(int num_orientations=9, int cell_size=8, int norm=0);
  /**
   * Destructor
   */
  ~HOG();
  /**
   * Compute feature
   *
   * @param img  image data (row-major)
   * @param w    image width
   * @param h    image height
   *
   * @return dimensionality
   */
  int Process(float *img, int w, int h);
  /**
   * Store features in an array of appropriate number of dimensions
   *
   * @param features  array where the data will be stored
   */
  void Extract(float *arr);
  /**
   * Return feature dimensionality
   *
   * @return  number of HOG dimensions
   */
  int ndim(void) const;

 private:

  int num_orientations_;  /**< number of orientation bins */
  int cell_size_;  /**< cell size */
  int norm_;  /**< Lp-normalization (0, 1 or 2) */

  void* hog_;  /**< VLFeat HOG object (defined as void to avoid linking issues) */
};

// for the Python interface with ctypes

#ifdef _PYTHON
extern "C" {
  HOG* HOG_new(int num_orientations, int cell_size, int norm);
  void HOG_del(HOG* obj);
  int HOG_Process(HOG* obj, float *img, int w, int h);
  void HOG_Extract(HOG* obj, float *arr);
}
#endif
#endif  /* LIB_CV_HOG_ */
