#ifndef LIB_ML_BERNOULLI_
#define LIB_ML_BERNOULLI_

#include "exponential_family.h"

// type-mapper for the derived class
class Bernoulli;
template<> struct ExponentialFamilyTSample<Bernoulli> {
  typedef uint8_t type;
};

const Real kDefBernoulliMinPHit = Real(1e-4);

class Bernoulli: public ExponentialFamily<uint8_t> {
 public:
  explicit Bernoulli(int nvar);

  virtual ~Bernoulli();

  virtual void SuffStat(const uint8_t* x, Real* t_out);

  virtual void FitFromSuffStat(const Real* sstat, const Real n = Real(1.0));

  virtual void Sample(uint8_t* x, const int n) const;

  Real min_phit(void);

  void set_min_phit(const Real min_phit);

  virtual Real LogH(const uint8_t* x);

  const Real* phit(void);

 private:
  Real min_phit_;
  Real* phit_;

  virtual void validate_eta(void);

  virtual void set_source_param(void);

  virtual void set_psi(void);

  virtual void set_grad_psi(void);
};

#ifdef _PYTHON
#include "exponential_family-inl.h"
EXPONENTIAL_FAMILY_DECLARE_CFN(Bernoulli)
extern "C" {
  Real Bernoulli_min_phit(Bernoulli* obj);
  void Bernoulli_set_min_phit(Bernoulli* obj, const Real min_phit);
  void Bernoulli_phit(Bernoulli* obj, Real* phit);
}
#endif

#endif  /* LIB_ML_BERNOULLI_ */
