#ifndef LIB_ML_DIRICHLET_
#define LIB_ML_DIRICHLET_

#include "exponential_family.h"

// type-mapper for the derived class
class Dirichlet;
template<> struct ExponentialFamilyTSample<Dirichlet> {
  typedef Real type;
};

class Dirichlet: public ExponentialFamily<Real> {
 public:
  explicit Dirichlet(int nvar);

  virtual ~Dirichlet();

  virtual void SuffStat(const Real* x, Real* t_out);

  virtual void FitFromSuffStat(const Real* sstat, const Real n = Real(1.0));

  virtual void Sample(Real* x, const int n) const;

  virtual Real LogH(const Real* x);

  const Real* alpha(void);

 private:
  Real *alpha_;

  virtual void validate_eta(void);

  virtual void set_source_param(void);

  virtual void set_psi(void);

  virtual void set_grad_psi(void);
};

#ifdef _PYTHON
#include "exponential_family-inl.h"
EXPONENTIAL_FAMILY_DECLARE_CFN(Dirichlet)
extern "C" {
  void Dirichlet_alpha(Dirichlet* obj, Real* alpha);
}
#endif

#endif  /* LIB_ML_DIRICHLET_ */
