#ifndef LIB_ML_EXPONENTIAL_FAMILY_INL_H_
#define LIB_ML_EXPONENTIAL_FAMILY_INL_H_

#include <cstring>

// This big and nasty macro declares all functions needed by the python wrapper.

// this must be called from the .h
#define EXPONENTIAL_FAMILY_DECLARE_CFN(DISTR)                           \
  typedef ExponentialFamilyTSample<DISTR>::type DISTR##_TSample;        \
  extern "C" {                                                          \
    DISTR* DISTR##_new(int nvar);                                       \
    void DISTR##_del(DISTR* obj);                                       \
    void DISTR##_eta(DISTR* obj, Real *eta);                            \
    void DISTR##_set_eta(DISTR* obj, const Real* eta);                  \
    void DISTR##_SuffStat(DISTR* obj, DISTR##_TSample* x, Real* t_out); \
    void DISTR##_FitFromSuffStat(DISTR* obj, const Real* sstat,         \
                                 const Real n);                         \
    Real DISTR##_psi(DISTR* obj);                                       \
    void DISTR##_grad_psi(DISTR* obj, Real* grad_psi);                  \
    Real DISTR##_LogLikelihood(DISTR* obj, const DISTR##_TSample* x);   \
    void DISTR##_Score(DISTR* obj, const DISTR##_TSample* x, int nsamples, Real* sx); \
    void DISTR##_Sample(DISTR* obj, DISTR##_TSample* x, const int n);   \
    int DISTR##_nvar(DISTR* obj);                                       \
    int DISTR##_npar(DISTR* obj);                                       \
    void DISTR##_Seed(DISTR* obj, const int s);                         \
  }                                                                     \

// this must be called from the .cc
#define EXPONENTIAL_FAMILY_IMPLEMENT_CFN(DISTR)                         \
  DISTR* DISTR##_new(int nvar) {                                        \
    return new DISTR(nvar);                                             \
  }                                                                     \
                                                                        \
  void DISTR##_del(DISTR* obj) {                                        \
    delete obj;                                                         \
  }                                                                     \
                                                                        \
  void DISTR##_eta(DISTR* obj, Real *eta) {                             \
    memcpy(eta, (Real*)obj->eta(), obj->npar() * sizeof(Real));         \
  }                                                                     \
                                                                        \
  void DISTR##_set_eta(DISTR* obj, const Real* eta) {                   \
    return obj->set_eta(eta);                                           \
  }                                                                     \
                                                                        \
  void DISTR##_SuffStat(DISTR* obj, DISTR##_TSample* x, Real* t_out) {  \
    return obj->SuffStat(x, t_out);                                     \
  }                                                                     \
                                                                        \
  void DISTR##_FitFromSuffStat(DISTR* obj, const Real* sstat,           \
                               const Real n) {                          \
    return obj->FitFromSuffStat(sstat, n);                              \
  }                                                                     \
                                                                        \
  Real DISTR##_psi(DISTR* obj) {                                        \
    return obj->psi();                                                  \
  }                                                                     \
                                                                        \
  void DISTR##_grad_psi(DISTR* obj, Real* grad_psi) {                   \
    memcpy(grad_psi, (Real*)obj->grad_psi(), obj->npar()*sizeof(Real)); \
  }                                                                     \
                                                                        \
  Real DISTR##_LogLikelihood(DISTR* obj, const DISTR##_TSample* x) {    \
    return obj->LogLikelihood(x);                                       \
  }                                                                     \
                                                                        \
  void DISTR##_Score(DISTR* obj, const DISTR##_TSample* x, int nsamples, Real* sx) { \
    return obj->Score(x, nsamples, sx);                                 \
  }                                                                     \
                                                                        \
  void DISTR##_Sample(DISTR* obj, DISTR##_TSample* x, const int n) {    \
    return obj->Sample(x, n);                                           \
  }                                                                     \
                                                                        \
  int DISTR##_npar(DISTR* obj) {                                        \
    return obj->npar();                                                 \
  }                                                                     \
                                                                        \
  int DISTR##_nvar(DISTR* obj) {                                        \
    return obj->nvar();                                                 \
  }                                                                     \
                                                                        \
  void DISTR##_Seed(DISTR* obj, const int s) {                          \
    return obj->Seed(s);                                                \
  }                                                                     \

#endif  // LIB_ML_EXPONENTIAL_FAMILY_INL_H_
