#ifndef LIB_ML_EXPONENTIAL_FAMILY_
#define LIB_ML_EXPONENTIAL_FAMILY_

#include "../core/core.h"

template<class TFamily>
struct ExponentialFamilyTSample;

template <class TFamily>
class ExponentialFamilyMixture;

template <typename T>
class ExponentialFamily {
  template <class TFamily> friend class ExponentialFamilyMixture;

 public:
  ExponentialFamily(int nvar, int npar);

  // TBD @ derived class
  virtual ~ExponentialFamily();

  // TBD @ derived class
  virtual void SuffStat(const T* x, Real* t_out) = 0;

  // TBD @ derived class
  virtual void FitFromSuffStat(const Real* sstat, const Real n = Real(1.0)) = 0;

  // TBD @ derived class
  virtual void Sample(T* x, const int n) const = 0;

  Real LogLikelihood(const T* x);

  void Score(const T* X, int nsamples, Real* scr);

  int npar(void) const;

  int nvar(void) const;

  const Real* eta(void) const;

  void set_eta(const Real *eta);

  Real psi(void) const;

  const Real* grad_psi(void) const;

  void Seed(const int s);

 protected:
  int nvar_;
  int npar_;
  Real* eta_;
  Real psi_;
  Real* grad_psi_;

  Real* t_buff_;

  // TBD @ derived class
  virtual Real LogH(const T* x) = 0;

  // TBD @ derived class
  virtual void validate_eta(void) = 0;

  // TBD @ derived class
  virtual void set_source_param(void) = 0;

  // TBD @ derived class
  virtual void set_psi(void) = 0;

  // TBD @ derived class
  virtual void set_grad_psi(void) = 0;

  void* gen_;
};

#endif  /* LIB_ML_EXPONENTIAL_FAMILY_ */
