#ifndef LIB_ML_EXPONENTIAL_FAMILY_MIXTURE_INL_H_
#define LIB_ML_EXPONENTIAL_FAMILY_MIXTURE_INL_H_

// This big and nasty macro declares all functions needed by the python wrapper.

// this must be called from the .h
#define EXPONENTIAL_FAMILY_MIXTURE_DECLARE_CFN(MIXTURE)                 \
  extern "C" {                                                          \
    MIXTURE* MIXTURE##_new(int ncomp, int nvar);                        \
    void MIXTURE##_del(MIXTURE* obj);                                   \
    void MIXTURE##_weights(MIXTURE* obj, Real* weights);                \
    void MIXTURE##_set_weights(MIXTURE* obj, const Real* weights);      \
    void MIXTURE##_eta(MIXTURE* obj, int k, Real* eta);                 \
    void MIXTURE##_set_eta(MIXTURE* obj, int k, const Real* eta);       \
    Real MIXTURE##_min_pst(MIXTURE* obj);                               \
    void MIXTURE##_set_min_pst(MIXTURE* obj, const Real min_pst);       \
    int MIXTURE##_npar(MIXTURE* obj);                                   \
    int MIXTURE##_nvar(MIXTURE* obj);                                   \
    void MIXTURE##_Sample(MIXTURE* obj, MIXTURE::TSample* x, const int n); \
    void MIXTURE##_Score(MIXTURE* obj, const MIXTURE::TSample* x,       \
                         int nsamples, Real* s_out, bool hard);         \
    void MIXTURE##_Fit(MIXTURE* obj, const MIXTURE::TSample* X,         \
                       int nsamples, Real tol, int max_iter,            \
                       Real alpha, Real beta);                          \
    Real MIXTURE##_Posteriors(MIXTURE* obj, const MIXTURE::TSample* x,  \
                              Real* pst);                               \
    void MIXTURE##_Seed(MIXTURE* obj, const int s);                     \
  }                                                                     \

// this must be called from the .cc
#define EXPONENTIAL_FAMILY_MIXTURE_IMPLEMENT_CFN(MIXTURE)               \
  MIXTURE* MIXTURE##_new(int ncomp, int nvar) {                         \
    return new MIXTURE(ncomp, nvar);                                    \
  }                                                                     \
                                                                        \
  void MIXTURE##_del(MIXTURE* obj) {                                    \
    delete obj;                                                         \
  }                                                                     \
                                                                        \
  void MIXTURE##_weights(MIXTURE* obj, Real* weights) {                 \
    memcpy(weights, obj->weights(), obj->ncomp() * sizeof(Real));       \
  }                                                                     \
                                                                        \
  void MIXTURE##_set_weights(MIXTURE* obj, const Real* weights) {       \
    obj->set_weights(weights);                                          \
  }                                                                     \
                                                                        \
  void MIXTURE##_eta(MIXTURE* obj, int k, Real* eta) {                  \
    const int npar = (*obj)[k]->npar();                                 \
    memcpy(eta, obj->eta(k), npar * sizeof(Real));                      \
  }                                                                     \
                                                                        \
  void MIXTURE##_set_eta(MIXTURE* obj, int k, const Real* eta) {        \
    obj->set_eta(k, eta);                                               \
  }                                                                     \
                                                                        \
  int MIXTURE##_npar(MIXTURE* obj) {                                    \
    return obj->npar();                                                 \
  }                                                                     \
                                                                        \
  int MIXTURE##_nvar(MIXTURE* obj) {                                    \
    return obj->nvar();                                                 \
  }                                                                     \
                                                                        \
  Real MIXTURE##_min_pst(MIXTURE* obj) {                                \
    return obj->min_pst();                                              \
  }                                                                     \
                                                                        \
  void MIXTURE##_set_min_pst(MIXTURE* obj, const Real min_pst) {        \
    return obj->set_min_pst(min_pst);                                   \
  }                                                                     \
                                                                        \
  void MIXTURE##_Sample(MIXTURE* obj, MIXTURE::TSample* x, const int n) { \
    return obj->Sample(x, n);                                           \
  }                                                                     \
                                                                        \
  void MIXTURE##_Score(MIXTURE* obj, const MIXTURE::TSample* x,         \
                       int nsamples, Real* s_out, bool hard) {          \
    return obj->Score(x, nsamples, s_out, hard);                        \
  }                                                                     \
                                                                        \
  void MIXTURE##_Fit(MIXTURE* obj, const MIXTURE::TSample* X,           \
                     int nsamples, Real tol, int max_iter,              \
                     Real alpha, Real beta) {                           \
    obj->Fit(X, nsamples, tol, max_iter, alpha, beta);                  \
  }                                                                     \
                                                                        \
  Real MIXTURE##_Posteriors(MIXTURE* obj, const MIXTURE::TSample* x,    \
                            Real* pst) {                                \
    return obj->Posteriors(x, pst);                                     \
  }                                                                     \
                                                                        \
  void MIXTURE##_Seed(MIXTURE* obj, int s) {                            \
    return obj->Seed(s);                                                \
  }                                                                     \

#endif  // LIB_ML_EXPONENTIAL_FAMILY_MIXTURE_INL_H_
