#ifndef LIB_ML_EXPONENTIAL_FAMILY_MIXTURE_
#define LIB_ML_EXPONENTIAL_FAMILY_MIXTURE_

#include "exponential_family.h"

const Real kDefExponentialFamilyMixtureMinPst = Real(1e-4);

template <class TFamily>
class ExponentialFamilyMixture {
 public:
  typedef typename ExponentialFamilyTSample<TFamily>::type TSample;

  explicit ExponentialFamilyMixture(int ncomp);

  virtual ~ExponentialFamilyMixture();

  Real Posteriors(const TSample *x, Real *pst);

  const TFamily* operator[](int k) const;

  TFamily* operator[](int k);

  const TFamily** components(void) const;

  const Real* weights(void) const;

  void set_weights(const Real* weights);

  const Real* eta(const int k) const;

  void set_eta(const int k, const Real* eta);

  Real min_pst(void) const;

  void set_min_pst(const Real min_pst);

  int ncomp(void) const;

  int npar(void) const;

  int nvar(void) const;

  void Sample(TSample* x, const int n);

  void Score(const TSample* X, int nsamples, Real* s_out, bool hard=false);

  void Fit(const TSample *X, int nsamples, Real tol = 1e-3, int max_iter = 100,
           Real alpha = 100.0, Real beta = 0.01);

  void Seed(const int s);

 protected:
  int ncomp_;
  Real min_pst_;
  TFamily** components_;
  Real* weights_;
  Real* log_weights_;
  Real* pst_buff_;
  Real* t_buff_;
  int* nz_indx_;

  void Init(void);

  Real LSE(const TSample *x, Real *log_posteriors);

  bool HandleEmpty(Real *pst_acc, Real beta);

  void* gen_;
};

#endif  /* LIB_ML_EXPONENTIAL_FAMILY_MIXTURE_ */
