#ifndef LIB_ML_FISHER_VECTOR_INL_H_
#define LIB_ML_FISHER_VECTOR_INL_H_

// This big and nasty macro declares all functions needed by the python wrapper.

// this must be called from the .h
#define FISHER_VECTOR_DECLARE_CFN(MIXTURE, TSAMPLE)                     \
  class MIXTURE;  /* mixture distribution forward declaration */        \
  typedef FisherVector<MIXTURE> MIXTURE##FisherVector;                  \
  typedef TSAMPLE MIXTURE##TSample;                                     \
  extern "C" {                                                          \
    FisherVector<MIXTURE>* MIXTURE##FisherVector_new(MIXTURE* model, int normalizer, Real theta, bool hard, Real alpha, int norm); \
    void MIXTURE##FisherVector_del(MIXTURE##FisherVector *obj);         \
    int MIXTURE##FisherVector_nL(MIXTURE##FisherVector* obj);           \
    void MIXTURE##FisherVector_L(MIXTURE##FisherVector* obj, Real* L);  \
    void MIXTURE##FisherVector_set_L(MIXTURE##FisherVector* obj, const Real* L); \
    Real MIXTURE##FisherVector_theta(MIXTURE##FisherVector* obj);  \
    void MIXTURE##FisherVector_set_theta(MIXTURE##FisherVector* obj, const Real val); \
    void MIXTURE##FisherVector_Fit(MIXTURE##FisherVector* obj, const int nsamples); \
    void MIXTURE##FisherVector_Process(MIXTURE##FisherVector* obj, const MIXTURE##TSample* X, int nsamples, Real* efv); \
  }                                                                     \

// this must be called from the .cc
#define FISHER_VECTOR_IMPLEMENT_CFN(MIXTURE)                            \
  MIXTURE##FisherVector* MIXTURE##FisherVector_new(MIXTURE* model, int normalizer, Real theta, bool hard, Real alpha, int norm) { \
    return new MIXTURE##FisherVector(model, normalizer, theta, hard, alpha, norm); \
  }                                                                     \
                                                                        \
  void MIXTURE##FisherVector_del(MIXTURE##FisherVector *obj) {          \
    delete obj;                                                         \
  }                                                                     \
                                                                        \
  void MIXTURE##FisherVector_L(MIXTURE##FisherVector* obj, Real* L) {   \
    if (obj->L() != NULL) {                                             \
      memcpy(L, (Real*)obj->L(), obj->nL() * sizeof(Real));             \
    }                                                                   \
  }                                                                     \
                                                                        \
  void MIXTURE##FisherVector_set_L(MIXTURE##FisherVector* obj, const Real* L) { \
    return obj->set_L(L);                                               \
  }                                                                     \
                                                                        \
  int MIXTURE##FisherVector_nL(MIXTURE##FisherVector* obj) {            \
    return obj->nL();                                                   \
  }                                                                     \
                                                                        \
  Real MIXTURE##FisherVector_theta(MIXTURE##FisherVector* obj) {        \
    return obj->theta();                                                \
  }                                                                     \
                                                                        \
  void MIXTURE##FisherVector_set_theta(MIXTURE##FisherVector* obj, const Real val) { \
    return obj->set_theta(val);                                         \
  }                                                                     \
                                                                        \
  void MIXTURE##FisherVector_Fit(MIXTURE##FisherVector* obj, const int nsamples) { \
    return obj->Fit(nsamples);                                          \
  }                                                                     \
                                                                        \
  void MIXTURE##FisherVector_Process(MIXTURE##FisherVector* obj, const MIXTURE##TSample* X, int nsamples, Real* efv) { \
    return obj->Process(X, nsamples, efv);                             \
  }                                                                     \
                                                                        \

#endif  // LIB_ML_FISHER_VECTOR_INL_H_
