#ifndef LIB_ML_FISHER_VECTOR_
#define LIB_ML_FISHER_VECTOR_

#include "../core/core.h"

enum {
  EYE=0,
  DIAG,
  VAR,
  BLOCK,
  FULL
};

const Real kShrinkage = Real(0.001);  // covariance estimate with shrinkage

template <class TMixture>
class FisherVector {
 public:
  typedef typename TMixture::TSample TSample;

  explicit FisherVector(TMixture* model, int normalizer=DIAG, Real theta=0.0, bool hard=false, Real alpha=0.5, int norm=2);

  virtual ~FisherVector();

  const Real* L(void) const;

  void set_L(const Real *L);

  int nL() const;

  Real theta(void);

  void set_theta(const Real val);

  void Fit(const int nsamples = int(1e5));

  void Process(const TSample* X, const int nsamples, Real *efv);

 protected:
  TMixture* model_;
  int normalizer_;
  Real theta_;
  bool hard_;
  Real alpha_;
  int norm_;
  Real* L_;
  void fit_var(const int nsamples);
  void fit_diag(const int nsamples);
  void fit_block(const int nsamples);
  void fit_full(const int nsamples);
};

// ctypes python wrappers
#ifdef _PYTHON
#include "fisher_vector-inl.h"
FISHER_VECTOR_DECLARE_CFN(GaussianMixture, Real)  // GaussianMixtureFisherVector
FISHER_VECTOR_DECLARE_CFN(BernoulliMixture, uint8_t)  // BernoulliMixtureFisherVector
FISHER_VECTOR_DECLARE_CFN(LaplaceMixture, Real)
FISHER_VECTOR_DECLARE_CFN(WishartMixture, Real)
FISHER_VECTOR_DECLARE_CFN(PoissonMixture, uint32_t)
FISHER_VECTOR_DECLARE_CFN(DirichletMixture, Real)
FISHER_VECTOR_DECLARE_CFN(MultinomialMixture, uint32_t)
#endif

#endif  /* LIB_ML_FISHER_VECTOR_ */
