#ifndef LIB_ML_GAUSSIAN_
#define LIB_ML_GAUSSIAN_

#include "exponential_family.h"

// type-mapper for the derived class
class Gaussian;
template<> struct ExponentialFamilyTSample<Gaussian> {
  typedef Real type;
};

const Real kDefGaussianMinVar = Real(1e-7);

class Gaussian: public ExponentialFamily<Real> {
 public:
  explicit Gaussian(int nvar);

  virtual ~Gaussian();

  virtual void SuffStat(const Real* x, Real* t_out);

  virtual void FitFromSuffStat(const Real* sstat, const Real n = Real(1.0));

  virtual void Sample(Real* x, const int n) const;

  Real min_var(void);

  void set_min_var(const Real min_var);

  virtual Real LogH(const Real* x);

  const Real* mu(void);

  const Real* sigma(void);

 private:
  Real min_var_;
  Real kLogH_;

  Real* mu_;
  Real* sigma_;

  virtual void validate_eta(void);

  virtual void set_source_param(void);

  virtual void set_psi(void);

  virtual void set_grad_psi(void);
};

#ifdef _PYTHON
#include "exponential_family-inl.h"
EXPONENTIAL_FAMILY_DECLARE_CFN(Gaussian)
extern "C" {
  Real Gaussian_min_var(Gaussian* obj);
  void Gaussian_set_min_var(Gaussian* obj, const Real min_var);
  void Gaussian_mu(Gaussian* obj, Real* mu);
  void Gaussian_sigma(Gaussian* obj, Real* sigma);
}
#endif

#endif  /* LIB_ML_GAUSSIAN_ */
