#ifndef LIB_ML_KMEANS_
#define LIB_ML_KMEANS_

#include "../core/core.h"

class KMeans {
 public:
  explicit KMeans(int K, int ndim, int max_iter=100, Real epsilon=Real(1e-2));

  ~KMeans();

  Real Fit(const Real* X, int nsamples);

  const Real* centroids(void) const;

  void set_centroids(const Real* centroids);

  const int* labels(void) const;

  int K(void);

  int ndim(void);

  Real Quantize(Real* X, int nsamples);

  Real Encode(const Real* X, int nsamples);

 protected:
  int K_;
  int ndim_;
  int max_iter_;
  Real epsilon_;
  Real *centroids_;
  int *labels_;
  int n_labels_;

  Real Init(const Real* X, int nsamples);
  Real RunOneIter(const Real* X, int nsamples);
};

#ifdef _PYTHON
extern "C" {
Real kmeans_fit(int K, int ndim, int max_iter, Real epsilon, const Real* X, int nsamples, Real* centroids);
Real kmeans_quantize(int K, int ndim, Real* centroids, Real* X, int nsamples);
Real kmeans_encode(int K, int ndim, Real* centroids, Real* X, int nsamples, int* q);
// KMeans* KMeans_new(int K, int ndim, int max_iter, Real epsilon);
// void KMeans_del(KMeans* model);
// Real KMeans_Fit(KMeans* model, const Real* X, int nsamples);
// void KMeans_centroids(KMeans* model, Real* centroids);
// Real KMeans_Quantize(KMeans* model, Real* X, int nsamples)
// Real KMeans_Encode(KMeans* model, Real* X, int nsamples, int* q)
}
#endif

#endif  /* LIB_ML_KMEANS_ */
