#ifndef LIB_ML_LAPLACE_
#define LIB_ML_LAPLACE_

#include "exponential_family.h"

// type-mapper for the derived class
class Laplace;
template<> struct ExponentialFamilyTSample<Laplace> {
  typedef Real type;
};

const Real kDefLaplaceMinScale = Real(1e-7);

class Laplace: public ExponentialFamily<Real> {
 public:
  explicit Laplace(int nvar);

  virtual ~Laplace();

  virtual void SuffStat(const Real* x, Real* t_out);

  virtual void FitFromSuffStat(const Real* sstat, const Real n = Real(1.0));

  virtual void Sample(Real* x, const int n) const;

  Real min_scale(void);

  void set_min_scale(const Real min_scale);

  virtual Real LogH(const Real* x);

  const Real* scale(void);

 private:
  Real min_scale_;
  Real *scale_;

  virtual void validate_eta(void);

  virtual void set_source_param(void);

  virtual void set_psi(void);

  virtual void set_grad_psi(void);
};

#ifdef _PYTHON
#include "exponential_family-inl.h"
EXPONENTIAL_FAMILY_DECLARE_CFN(Laplace)
extern "C" {
  Real Laplace_min_scale(Laplace* obj);
  void Laplace_set_min_scale(Laplace* obj, const Real min_scale);
  void Laplace_scale(Laplace* obj, Real* scale);
}
#endif

#endif  /* LIB_ML_LAPLACE_ */
