#ifndef LIB_ML_MULTINOMIAL_
#define LIB_ML_MULTINOMIAL_

#include "exponential_family.h"

// type-mapper for the derived class
class Multinomial;
template<> struct ExponentialFamilyTSample<Multinomial> {
  typedef uint32_t type;
};

const Real kDefMultinomialMinP = Real(1e-7);

class Multinomial: public ExponentialFamily<uint32_t> {
 public:
  explicit Multinomial(int nvar);

  virtual ~Multinomial();

  virtual void SuffStat(const uint32_t* x, Real* t_out);

  virtual void FitFromSuffStat(const Real* sstat, const Real n = Real(1.0));

  virtual void Sample(uint32_t* x, const int n) const;

  uint32_t ntrials(void);

  void set_ntrials(const uint32_t ntrials);

  Real min_p(void);

  void set_min_p(const Real min_p);

  virtual Real LogH(const uint32_t* x);

  const Real* p(void);

 private:
  Real ntrials_;
  Real log_ntrials_factorial_;
  Real min_p_;
  Real* p_;

  virtual void validate_eta(void);

  virtual void set_source_param(void);

  virtual void set_psi(void);

  virtual void set_grad_psi(void);
};

#ifdef _PYTHON
#include "exponential_family-inl.h"
EXPONENTIAL_FAMILY_DECLARE_CFN(Multinomial)
extern "C" {
  uint32_t Multinomial_ntrials(Multinomial* obj);
  void Multinomial_set_ntrials(Multinomial* obj, const uint32_t ntrials);
  Real Multinomial_min_p(Multinomial* obj);
  void Multinomial_set_min_p(Multinomial* obj, const Real min_p);
  void Multinomial_p(Multinomial* obj, Real* p);
}
#endif

#endif  /* LIB_ML_MULTINOMIAL_ */
