#include "multinomial_mixture.h"

MultinomialMixture::MultinomialMixture(int ncomp, int nvar)
    :ExponentialFamilyMixture<Multinomial>::ExponentialFamilyMixture(ncomp) {
  for (int k = 0; k < ncomp_; ++k) {
    components_[k] = new Multinomial(nvar);
  }
}

MultinomialMixture::~MultinomialMixture() {
}

uint32_t
MultinomialMixture::ntrials(void) {
  assert(ncomp_ > 0);
  return components_[0]->ntrials();
}

void
MultinomialMixture::set_ntrials(const uint32_t ntrials) {
  for (int k = 0; k < ncomp_; ++k) {
    components_[k]->set_ntrials(ntrials);
  }
}

Real
MultinomialMixture::min_p(void) {
  assert(ncomp_ > 0);
  return components_[0]->min_p();
}

void
MultinomialMixture::set_min_p(const Real min_p) {
  for (int k = 0; k < ncomp_; ++k) {
    components_[k]->set_min_p(min_p);
  }
}

#ifdef _PYTHON
EXPONENTIAL_FAMILY_MIXTURE_IMPLEMENT_CFN(MultinomialMixture)
uint32_t MultinomialMixture_ntrials(MultinomialMixture* obj) {
  return obj->ntrials();
}
void MultinomialMixture_set_ntrials(MultinomialMixture* obj, const uint32_t ntrials) {
  return obj->set_ntrials(ntrials);
}
Real MultinomialMixture_min_p(MultinomialMixture* obj) {
  return obj->min_p();
}
void MultinomialMixture_set_min_p(MultinomialMixture* obj, const Real min_p) {
  return obj->set_min_p(min_p);
}
#endif
