#ifndef LIB_ML_POISSON_
#define LIB_ML_POISSON_

#include "exponential_family.h"

#include <cmath>

// type-mapper for the derived class
class Poisson;
template<> struct ExponentialFamilyTSample<Poisson> {
  typedef uint32_t type;
};

const Real kDefPoissonMinLambda = Real(1e-7);

class Poisson: public ExponentialFamily<uint32_t> {
 public:
  explicit Poisson(int nvar);

  virtual ~Poisson();

  virtual void SuffStat(const uint32_t* x, Real* t_out);

  virtual void FitFromSuffStat(const Real* sstat, const Real n = Real(1.0));

  virtual void Sample(uint32_t* x, const int n) const;

  Real min_lambda(void);

  void set_min_lambda(const Real min_lambda);

  virtual Real LogH(const uint32_t* x);

  const Real* lambda(void);

 private:
  Real min_lambda_;
  Real *lambda_;

  virtual void validate_eta(void);

  virtual void set_source_param(void);

  virtual void set_psi(void);

  virtual void set_grad_psi(void);
};

#ifdef _PYTHON
#include "exponential_family-inl.h"
EXPONENTIAL_FAMILY_DECLARE_CFN(Poisson)
extern "C" {
  Real Poisson_min_lambda(Poisson* obj);
  void Poisson_set_min_lambda(Poisson* obj, const Real min_lambda);
  void Poisson_lambda(Poisson* obj, Real* lambda);
}
#endif

#endif  /* LIB_ML_POISSON_ */
