#ifndef LIB_ML_WISHART_H_
#define LIB_ML_WISHART_H_

#include "exponential_family.h"

// type-mapper for the derived class
class Wishart;
template<> struct ExponentialFamilyTSample<Wishart> {
  typedef Real type;
};

/*
 * Wishart distribution with fixed degrees of freedom.
 * Based on:
 * Saint-Jean, Christophe, and Frank Nielsen. "A new implementation of k-MLE
 * for mixture modeling of Wishart distributions." Geometric Science of
 * Information. Springer Berlin Heidelberg, 2013. 249-256
*/
class Wishart: public ExponentialFamily<Real> {
 public:
  explicit Wishart(int nvar);

  virtual ~Wishart();

  virtual void SuffStat(const Real* x, Real* t_out);

  virtual void FitFromSuffStat(const Real* sstat, const Real n = Real(1.0));

  virtual void Sample(Real* x, const int n) const;

  Real dof(void);

  void set_dof(const Real dof);

  virtual Real LogH(const Real* x);

  const Real* V(void);

  const Real* L(void);

 private:
  int ndim_;  // sqrt(nvar)
  Real dof_;
  Real* V_;
  Real* L_;

  virtual void validate_eta(void);

  virtual void set_source_param(void);

  virtual void set_psi(void);

  virtual void set_grad_psi(void);
};

#ifdef _PYTHON
#include "exponential_family-inl.h"
EXPONENTIAL_FAMILY_DECLARE_CFN(Wishart)
extern "C" {
  Real Wishart_dof(Wishart* obj);
  void Wishart_set_dof(Wishart* obj, const Real dof);
  void Wishart_V(Wishart* obj, Real* V);
  void Wishart_L(Wishart* obj, Real* L);
}
#endif

#endif  // LIB_ML_WISHART_H_
