#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
sys.path.append('.')

import unittest
import numpy as np

from skimage import io

from vrl.cv import DSIFT


class TestDSIFT(unittest.TestCase):
    def __init__(self, *args):
        unittest.TestCase.__init__(self, *args)

    def setUp(self):
        self.dsift_max_layer = 0
        self.dsift_step = 8
        self.dsift_size = 8
        self.dsift_downscale = 1.5
        self.dsift_norm = 2
        self.img = io.imread('tests/mandrill.jpg', as_grey=True)
        return

    def test_compute(self):
        dsift = DSIFT(max_layer=self.dsift_max_layer,
                      downscale=self.dsift_downscale,
                      step=self.dsift_step,
                      size=self.dsift_size,
                      norm=self.dsift_norm)

        data = dsift.transform(self.img)
        locs = data['loc']
        feats = data['data']
        self.assertEqual(locs[0][0], 12)
        self.assertEqual(locs[0][1], 12)
        self.assertEqual(locs[0][2], 1.0)
        self.assertAlmostEqual(np.linalg.norm(feats[0]), 1.0, places=5)
        self.assertEqual(len(feats), len(locs))
        return

if __name__ == '__main__':
    unittest.main()
