#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import unittest

import numpy as np

import sys
sys.path.append('.')
from vrl.ml import PCA
import vrl.io


class TestPCA(unittest.TestCase):
    def __init__(self, *args):
        unittest.TestCase.__init__(self, *args)

    def setUp(self):
        self.mean = np.array([0, 0])
        self.cov = np.array([[2, 0], [0, 1]])
        self.train_data = np.random.multivariate_normal(self.mean, self.cov,
                                                        1000)
        self.test_data = np.random.multivariate_normal(self.mean, self.cov,
                                                       1000)
        self.pca = PCA()
        self.pca.fit(self.train_data)
        return

    def test_init(self):
        pca = PCA()
        self.assertTrue(pca.mean_ is None)
        self.assertTrue(pca.sval_ is None)
        self.assertTrue(pca.svec_ is None)
        pca = PCA()
        pca.mean_ = 1.0
        pca.sval_ = 2.0
        pca.svec_ = 3.0
        self.assertEqual(pca.mean_, 1.0)
        self.assertEqual(pca.sval_, 2.0)
        self.assertEqual(pca.svec_, 3.0)

    def test_transform(self):
        self.pca.dim = 1
        test_data = self.pca.transform(self.test_data)
        self.assertEqual(test_data.shape, (len(self.test_data), 1))
        self.pca.dim = 2
        test_data = self.pca.transform(self.test_data)
        self.assertEqual(test_data.shape, (len(self.test_data), 2))
        return

    def test_read_write_model(self):
        filename = '/tmp/pca_model.npz'
        if vrl.io.file_exists(filename):
            os.remove(filename)
        vrl.io.save(filename, self.pca)
        self.assertTrue(vrl.io.file_exists(filename))
        return

if __name__ == '__main__':
    unittest.main()
