# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division

class BoundingBox(object):
    def __init__(self, x1, y1, x2, y2):
        self._x1 = float(x1)
        self._y1 = float(y1)
        self._x2 = float(x2)
        self._y2 = float(y2)
        self._area = (self._x2 - self._x1) * (self._y2 - self._y1)

    @property
    def x1(self):
        return self._x1

    @property
    def y1(self):
        return self._y1

    @property
    def x2(self):
        return self._x2

    @property
    def y2(self):
        return self._y2

    @property
    def area(self):
        return self._area

    def overlap(self, bb):
        if bb is self:
            return 1.0

        if self._x1 > bb._x2 or self._y1 > bb._y2 or \
           self._x2 < bb._x1 or self._y2 < bb._y1:
            return 0.0

        cint = min(self._x2, bb._x2) - max(self._x1, bb._x1)
        rint = min(self._y2, bb._y2) - max(self._y1, bb._y1)
        intersection = rint * cint
        return intersection / (self._area + bb._area - intersection)

    def __str__(self):
        return '{}(x1={}, y1={}, x2={}, y2={})'.format(self.__class__.__name__,
                                                       self._x1, self._y1,
                                                       self._x2, self._y2)