# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division

from .dataset import ClassificationDataset


class Brodatz32(ClassificationDataset):
    ''' Brodatz 32 textures dataset '''
    def __init__(self, path, random_state=None):
        '''
        Parameters
        ----------
        path:  dataset path
        random_state:  external RNG. If None, create its own
        '''
        ClassificationDataset.__init__(self, path=path,
                                       random_state=random_state)

        if len(self._imlist) != 2048:
            raise RuntimeError('Dataset corrupted. Wrong number of images')

        classes = ('bark', 'beachsand', 'beans', 'burlap', 'd10', 'd11', 'd4',
                   'd51', 'd52', 'd5', 'd6', 'd95', 'fieldstone', 'grass',
                   'ice', 'image09', 'image15', 'image17', 'image19', 'paper',
                   'peb54', 'pigskin', 'pressedcl', 'raffia2', 'raffia',
                   'reptile', 'ricepaper', 'seafan', 'straw2', 'tree', 'water',
                   'woodgrain')

        self._classes = tuple(sorted(classes))

        self.generate_split()

    def generate_split(self):
        labeled_samples = zip(self._imlist, self._labels_from_path('_'))
        self._random_state.shuffle(labeled_samples)
        self._samples['train'] = labeled_samples[:1024]
        self._samples['test'] = labeled_samples[1024:]
