# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division

from os.path import sep, join, pardir, abspath

from .dataset import ClassificationDataset


class IndoorScenes(ClassificationDataset):
    ''' MIT-CSAIL indoor scenes dataset '''
    def __init__(self, path, random_state=None):
        '''
        Parameters
        ----------
        path:  dataset path
        random_state:  external RNG. If None, create its own
        '''
        ClassificationDataset.__init__(self, path=path,
                                       random_state=random_state)

        if len(self._imlist) != 15620:
            raise RuntimeError('Dataset corrupted. Wrong number of images')

        classes = ('airport_inside', 'artstudio', 'auditorium', 'bakery',
                   'bar', 'bathroom', 'bedroom', 'bookstore', 'bowling',
                   'buffet', 'casino', 'children_room', 'church_inside',
                   'classroom', 'cloister', 'closet', 'clothingstore',
                   'computerroom', 'concert_hall', 'corridor', 'deli',
                   'dentaloffice', 'dining_room', 'elevator',
                   'fastfood_restaurant', 'florist', 'gameroom', 'garage',
                   'greenhouse', 'grocerystore', 'gym', 'hairsalon',
                   'hospitalroom', 'inside_bus', 'inside_subway',
                   'jewelleryshop', 'kindergarden', 'kitchen', 'laboratorywet',
                   'laundromat', 'library', 'livingroom', 'lobby',
                   'locker_room', 'mall', 'meeting_room', 'movietheater',
                   'museum', 'nursery', 'office', 'operating_room', 'pantry',
                   'poolinside', 'prisoncell', 'restaurant',
                   'restaurant_kitchen', 'shoeshop', 'stairscase',
                   'studiomusic', 'subway', 'toystore', 'trainstation',
                   'tv_studio', 'videostore', 'waitingroom', 'warehouse',
                   'winecellar')

        self._classes = tuple(sorted(classes))

        lstpath = abspath(join(self._impath, pardir))

        train_files = join(lstpath, 'TrainImages.txt')
        train_files = [f.strip() for f in open(train_files).readlines()]
        self._samples['train'] = zip(train_files, self._labels_from_path(sep, train_files))

        test_files = join(lstpath, 'TestImages.txt')
        test_files = [f.strip() for f in open(test_files).readlines()]
        self._samples['test'] = zip(test_files, self._labels_from_path(sep, test_files))
