# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division

from os.path import sep

from .dataset import ClassificationDataset


class KTH_TIPS2(ClassificationDataset):
    ''' KTH-TIPS2 textures dataset '''
    def __init__(self, path, sample='a', random_state=None):
        '''
        Parameters
        ----------
        path:  dataset path
        sample:  test set (\'a\', \'b\', \'c\' or \'d\')
        random_state:  external RNG. If None, create its own
        '''
        ClassificationDataset.__init__(self, path=path,
                                       random_state=random_state)

        if len(self._imlist) == 4608:
            self.logger.info('KTH-TIPS2a detected')
        elif len(self._imlist) == 4752:
            self.logger.info('KTH-TIPS2b detected')
        else:
            raise RuntimeError('Dataset corrupted. Wrong number of images')

        classes = ('aluminium_foil', 'brown_bread', 'corduroy', 'cork',
                   'cotton', 'cracker', 'lettuce_leaf', 'linen', 'white_bread', 'wood',
                   'wool')

        self._classes = tuple(sorted(classes))

        self.generate_split(sample=sample)

    def generate_split(self, sample='a'):
        if not sample in ('a', 'b', 'c', 'd'):  # 3 for training / 1 for evaluation (sample)
            raise ValueError('sample must be \'a\', \'b\', \'c\' or \'d\'')
        self._sample_id = 'sample_' + sample

        labeled_samples = zip(self._imlist, self._labels_from_path(sep))
        self._random_state.shuffle(labeled_samples)
        self._samples['train'] = [x for x in labeled_samples if self._sample_id not in x[0]]
        self._samples['test'] = [x for x in labeled_samples if self._sample_id in x[0]]
