# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division

import os
import scipy.io as sio
import numpy as np

from .dataset import Dataset


class OxfordFlowers(Dataset):
    ''' Oxford 102 Flowers dataset '''
    def __init__(self, path, test='val'):
        classes = tuple(range(1, 102+1))  # class names are IDs

        Dataset.__init__(self, path=path, classes=classes)

        assert(len(self._images) == 8189)

        self._read_labels()

        assert(test in ('val', 'test'))
        self._read_split(test=test)

    def _read_labels(self):
        base_path = os.path.abspath(os.path.join(self._path, os.path.pardir))
        labels_file = os.path.join(base_path, 'imagelabels.mat')

        labels = sio.loadmat(labels_file)['labels'].ravel()
        assert(len(self._images) == len(labels))

        self._samples = []
        for i, lbl in enumerate(labels):
            imfile = self._images[i]  # images are sorted by ID number
            cid = [self._classes.index(lbl)]
            self._samples.append({'file': imfile, 'class_id': tuple(cid)})

        self._generate_sets()

    def _read_split(self, test):
        # shuffle all the samples
        self._shuffle()

        base_path = os.path.abspath(os.path.join(self._path, os.path.pardir))
        setid_file = os.path.join(base_path, 'setid.mat')

        mat = sio.loadmat(setid_file)

        if test == 'val':
            train_id = mat['trnid'].ravel()
            test_id = mat['valid'].ravel()

        else:
            train_id = np.concatenate([mat['trnid'].ravel(),
                                       mat['valid'].ravel()])
            test_id = mat['tstid'].ravel()

        self._sets['train'] = []
        self._sets['test'] = []
        for sample in self._samples:
            id = 1 + self._images.index(sample['file'])
            if id in train_id:
                self._sets['train'].append(sample)
            elif id in test_id:
                self._sets['test'].append(sample)
        self._sets['train'] = tuple(self._sets['train'])
        self._sets['test'] = tuple(self._sets['test'])
