# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division

from os.path import sep

from .dataset import ClassificationDataset


class Scenes15(ClassificationDataset):
    ''' 15 Scenes dataset '''
    def __init__(self, path, n=100, random_state=None):
        '''
        Parameters
        ----------
        path:  dataset path
        n:  train/test using \'n\' images per class
        random_state:  external RNG. If None, create its own
        '''
        ClassificationDataset.__init__(self, path=path,
                                       random_state=random_state)

        if len(self._imlist) != 4485:
            raise RuntimeError('Dataset corrupted. Wrong number of images')

        classes = ('bedroom', 'CALsuburb', 'industrial', 'kitchen',
                   'livingroom', 'MITcoast', 'MITforest', 'MIThighway',
                   'MITinsidecity', 'MITmountain', 'MITopencountry',
                   'MITstreet', 'MITtallbuilding', 'PARoffice', 'store')

        self._classes = tuple(sorted(classes))

        self._n = int(n)
        if self._n < 1:
            raise ValueError('n must be a positive ineger')

        self.generate_split()

    def generate_split(self):
        samples = zip(self._imlist, self._labels_from_path(sep))
        train_set, test_set = self._n_per_class_split(self._n, -1, samples)
        self._samples['train'] = train_set
        self._samples['test'] = test_set
