# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division

from os.path import join, sep, exists, pardir

from ..io import load

from .dataset import ClassificationDataset


class SUN397(ClassificationDataset):
    ''' SUN397 dataset '''
    def __init__(self, path, split=1, random_state=None):
        '''
        Parameters
        ----------
        path:  dataset path
        n:  train/test using \'n\' images per class
        random_state:  external RNG. If None, create its own
        '''
        ClassificationDataset.__init__(self, path=path,
                                       random_state=random_state)

        if len(self._imlist) != 108754:
            raise RuntimeError('Dataset corrupted. Wrong number of images')

        # splits are in the "Partitions" subdir
        partitions_path = join(join(self._impath, pardir), 'Partitions')  # SUN397/../Partitions
        if not exists(partitions_path):
            partitions_path = join(self._impath, 'Partitions')  # SUN397/Partitions
            if not exists(partitions_path):
                raise IOError('\'Partitions\' path not found')

        class_name_file = join(partitions_path, 'ClassName.txt')
        classes = [lin.strip()[1:] for lin in open(class_name_file).readlines()]
        self._classes = tuple(sorted(classes))

        self._split = int(split)
        if not 1 <= self._split <= 10:
            raise ValueError('split is out of range')

        training_file = join(partitions_path, 'Training_{:02d}.txt'.format(self._split))
        training = [f.strip()[1:] for f in open(training_file).readlines()]
        self._samples['train'] = zip(training, self._labels_from_path(sep, training))

        testing_file = join(partitions_path, 'Testing_{:02d}.txt'.format(self._split))
        testing = [f.strip()[1:] for f in open(testing_file).readlines()]
        self._samples['test'] = zip(testing, self._labels_from_path(sep, testing))
