# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division

from os.path import sep

from .dataset import ClassificationDataset


class UIUC_Sports(ClassificationDataset):
    ''' UIUCSports dataset '''
    def __init__(self, path, random_state=None):
        '''
        Parameters
        ----------
        path:  dataset path
        random_state:  external RNG. If None, create its own
        '''
        ClassificationDataset.__init__(self, path=path,
                                       random_state=random_state)

        if len(self._imlist) != 1579:
            raise RuntimeError('Dataset corrupted. Wrong number of images')

        classes = ('badminton', 'bocce', 'croquet', 'polo', 'RockClimbing',
                   'rowing', 'sailing', 'snowboarding')

        self._classes = tuple(sorted(classes))

        self.generate_split()

    def generate_split(self):
        samples = zip(self._imlist, self._labels_from_path(sep))
        train_set, test_set = self._n_per_class_split(70, 60, samples)
        self._samples['train'] = train_set
        self._samples['test'] = test_set
