# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division

from os import makedirs
from os.path import join, basename, abspath, exists, dirname

import numpy as np

from .dataset import Dataset


class UKBench(Dataset):
    ''' UKBench dataset '''
    def __init__(self, path, random_state=None):
        '''
        Parameters
        ----------
        path:  dataset path
        random_state:  external RNG. If None, create its own
        '''
        Dataset.__init__(self, path=path, random_state=random_state)

        # remove images from the thumbnail path
        if len(self._imlist) == 20400:
            self._imlist = filter(lambda f: '.th.jpg' not in f, self._imlist)
            self._imlist = map(lambda f: basename(f), self._imlist)
            self._impath = join(self._impath, 'full')

        n_img = len(self._imlist)
        if not n_img == 10200:
            raise RuntimeError('Dataset corrupted. Wrong number of images')

        # self._imlist = self._imlist[:4*100]

        # stores samples as lsit list of (iamge, id) tuples
        self._samples = zip(self._imlist, [(i//4) for i in xrange(n_img)])

    def eval(self, scr, outfile=None):
        n_test = len(self._samples)
        if not isinstance(scr, np.ndarray) or scr.size != n_test**2:
            raise ValueError('src must be a {0}x{0} matrix'.format(n_test))

        top_indx = []
        top_count = []
        for i in xrange(n_test):
            id_i = self._samples[i][1]
            top_i = np.argsort(-scr[i, :])[:4].tolist()
            id_j = [self._samples[j][1] for j in top_i]
            top_count_i = int(id_j[0] == id_i) + int(id_j[1] == id_i) + \
                int(id_j[2] == id_i) + int(id_j[3] == id_i)
            top_count.append(top_count_i)
            top_indx.append(top_i)

        r4 = float(sum(top_count)) / float(n_test)
        r4 = '4 x recall@4: {:.3f}'.format(r4)

        if outfile is not None:
            info_str_ = []
            fmt = '{} {:5d} {:5d} {:5d} {:5d}  {:+.4f} {:+.4f} {:+.4f} {:+.4f}'
            for i, indx in enumerate(top_indx):
                str_ = fmt.format(top_count[i], indx[0], indx[1], indx[2],
                                  indx[3], scr[i, indx[0]], scr[i, indx[1]],
                                  scr[i, indx[2]], scr[i, indx[3]])
                info_str_.append(str_)

            if not exists(dirname(outfile)):
                makedirs(abspath(outfile))
            f = open(outfile, 'w')
            f.write('\n'.join(info_str_) + '\n')
            f.write(r4 + '\n')
            f.close()
        else:
            self.logger.info(r4)
