# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division

import numpy as np

from ..base import VRLBaseEstimator


class AsType(VRLBaseEstimator):
    def __init__(self, dtype=np.float64):
        VRLBaseEstimator.__init__(self)
        if isinstance(dtype, str):
            dtype = np.dtype(dtype).type
        self.dtype = dtype

    @property
    def dtype(self):
        return self._dtype

    @dtype.setter
    def dtype(self, val):
        self._dtype = val

    def get_params(self, deep=True):
        return {'dtype': np.dtype(self._dtype).name}

    def _fit(self, X, y=None):
        return

    def _transform(self, X):
        if isinstance(X, np.ndarray):
            return X.astype(self._dtype)
        elif isinstance(X, dict) and 'data' in X:
            _X = X.copy()
            _X['data'] = X['data'].astype(self._dtype)
            return _X