# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division

import numpy as np

from ..base import VRLBaseEstimator


class Binarizer(VRLBaseEstimator):
    ''' sample binarizer '''
    def __init__(self, method='zero'):
        VRLBaseEstimator.__init__(self)

        self.__transform_fn = {'zero': self.__transform_zero,
                               'mean': self.__transform_thresholds,
                               'median': self.__transform_thresholds}

        self.method = method

        self.thresholds_ = None

    @property
    def method(self):
        return self._dim

    @method.setter
    def method(self, val):
        if not val in self.__transform_fn.keys():
            raise ValueError
        self._method = val

    def get_params(self, deep=True):
        params = {'method': self._method}
        if self._method in ('mean', 'median',):
            params.update({'thresholds_': self.thresholds_})
        return params

    def _fit(self, X, y=None):
        if not isinstance(X, np.ndarray):
            raise TypeError

        if self._method == 'zero':
            return

        elif self._method == 'mean':
            self.thresholds_ = np.mean(X, axis=0)

        elif self._method == 'median':
            self.thresholds_ = np.median(X, axis=0)

        else:
            raise NotImplementedError

    def _transform(self, X):
        if isinstance(X, np.ndarray):
            data = X
        elif isinstance(X, dict) and 'data' in X:
            data = X['data']
        else:
            raise TypeError('wrong data type')

        bin_data = self.__transform_fn[self._method](data)
        bin_data = bin_data.astype(np.uint8)
        #bin_data = bin_data.astype(np.float64)

        # if dict, keep all other fields unchanged
        if isinstance(X, dict):
            _X = X.copy()
            _X['data'] = bin_data
            return _X

        return bin_data

    def __transform_zero(self, x):
        return (x > 0.)

    def __transform_thresholds(self, x):
        return (x[np.newaxis, :] > self.thresholds_)
