# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division

import numpy as np

from ctypes import cdll

from .dirichlet import Dirichlet
from .exponential_family_mixture import lib, ExponentialFamilyMixture


class DirichletMixture(ExponentialFamilyMixture):

    def __init__(self, ncomp, nvar, min_pst=1e-4, random_state=None):
        ''' Dirichlet mixture

        Parameters
        ----------
        ncomp:  number of mixture components
        nvar:  number of input dimensions
        min_pst:  posteriors below min_pst are set to zero
        '''
        ExponentialFamilyMixture.__init__(self, distr=Dirichlet, ncomp=ncomp,
                                          random_state=None, dtype=np.float64)

        # create object
        self._cobj = self._cfn_new(ncomp, nvar)

        # set optional parameters
        self.min_pst = min_pst

    def _empty_component(self):
        return Dirichlet(nvar=self.nvar, random_state=self._random_state)

    def compute_initial_estimate(self, data, **kwargs):
        ExponentialFamilyMixture._kmeans_initial_estimate(self, data, **kwargs)
