# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import division

import os

import numpy as np

from numpy.random import RandomState

from math import sqrt as msqrt

from utils import is_64bit


class RNG(RandomState):
    '''VRL Random Number Generator class'''
    def __init__(self, seed=None, **kwargs):
        RandomState.__init__(self, **kwargs)
        self.seed(seed=seed)

    def seed(self, seed=None):
        if seed is None:
            if(is_64bit()):
                seed = int(os.urandom(4).encode('hex'), 16)
                RandomState.seed(self, seed)
            else:
                seed = int(os.urandom(3).encode('hex'), 16)
                RandomState.seed(self, seed)
        else:
            if(is_64bit()):
                RandomState.seed(self, seed)
            else:
                RandomState.seed(self, abs(seed))

    def wishart(self, n, p, L=None):
        ''' generates a PxP Wishart sample with N degrees of freedom'''
        A = np.zeros((p, p))
        for i in xrange(p):
            A[i, i] = msqrt(self.gamma(0.5 * (n - i), 2.0))
            A[i, :i] = self.normal(size=(i, ))

        B = A if L is None else np.dot(L, A)

        # TODO: return a scipy.sparse.triu matrix
        return np.dot(B, B.T)
